<?php
/**
*
* WP_Malscart_HTML: a class for building and outputting HTML elements
*
* this is a static utility class
*
* we have public methods for building some form element types that are 
* actually several tags in a specific configuration:
* 	text_field
*	radio_field
*	dropdown_field
*	print_text_field
*	print_radio_field
*	print_dropdown_field
*
* and also generic methods:
*	build_tag
*	print_tag
*
*
* the $attributes array is structured thus:
*
* tag: the tag name
* text: whatever the tag is to wrap (only for wrapping tags)
* items: array for complex tags like radio buttons and selects
* attributes:(array)
*	value
* 	style
*	id
*	name
*	class
*	... (any html tag attributes you want)
* desc: used for help text
*
*/

class WP_Malscart_HTML {

	// tags that don't enclose content
	private static $self_closers = array('input','img','hr','br','meta','link');
	
	// reserved shortcode argument names
	private static $standard_atts = array( 'price', 'product', 'product[]', 'button_style', 'button_text', 'button_width' );
	
	private static $buy_button = true;

	// added to top-level html elements generated by the plugin	
	private static $base_class = 'wpmc';
	
	public function build_tag( $atts ) {
		return self::build( $atts );
	}
	
	public function print_tag( $atts ) {
		echo self::build( $atts );
	}
	
	public function print_radio_field( $atts ) {
		echo self::radio_field( $atts );
	}
	
	public function print_text_field( $atts ) {
		echo self::text_field( $atts );
	}
	
	public function print_dropdown_field( $atts ) {
		echo self::dropdown_field( $atts );
	}
	
	// build a text form field
	public function text_field( $atts ) {

		$atts[ 'tag' ] = 'input';
		$atts[ 'type' ] = 'text';

		if ( isset( $atts[ 'text' ] ) && ! empty( $atts[ 'text' ] ) ) // if we have text, set it as the label
			return self::build( array( 'tag'=>'label', 'text'=>$atts[ 'text' ].self::build( $atts ) ) ).$atts[ 'desc' ];

		else return self::build( $atts ).$atts[ 'desc' ];

	}
	
	/*
	* builds a radio button element (series of same-named checkboxes) 
	*/
	public function radio_field( $atts ) {
		
		$atts[ 'tag' ] = 'input';
		$atts[ 'type' ] = 'radio';
		
		$html = array();
		
		foreach( $atts['items'] as $label=>$element_value ) {
			$element_atts = $atts;
			if ( $atts[ 'value' ] == $element_value ) $element_atts[ 'checked' ] = 'checked';
			$element_atts[ 'value' ] = $element_value;
			$element_atts[ 'text' ] = $label;
			$html[] = self::build( array( 'tag'=>'label', 'text'=>self::build( $element_atts ).$label.'<br/>' ) );
		}

		return self::output( $html );
	}

	/*
	*
	* builds a dropdown field element
	*
	* expects a name for the field and an array of options in $label => $value format
	*
	*/
	public function  dropdown_field( $atts ) {

		// builds an array of option elements
		$option_fields = array();
		foreach( $atts['items'] as $label => $element_value ) {
			$element_atts = $atts;
			if ( $atts[ 'value' ] == $element_value ) $element_atts[ 'selected' ] = 'selected';
			$element_atts[ 'value' ] = $element_value;
			$element_atts[ 'text' ] = $label;
			$element_atts[ 'tag' ] = 'option';
			$option_fields[] = self::build( $element_atts );
		}
		
		// builds the select element and inserts the option fields into it
		$atts[ 'tag' ] = 'select';
		$atts[ 'text' ] = self::output( $option_fields );
		return self::build( $atts );

	}
	
	public function isAssoc( Array $array ) {
		
			return ( $array !== array_values( $array ) );
			
	}
	
	/*    BUY BUTTON OUTPUT FUNCTIONS      */
	
	/**
	* this is the shortcode function for outputting a button of any style
	* all the formats for the button styles are here
	* unless we come up with something more elegant, this will be a series of cases with procedural builds for each one
	* this function the buy_button shortcode calls
	*/
	public function buy_button( $atts ) {
		
		global $post;
		
		self::$buy_button = true;// this is a buy button, not an add button
		
		$atts = self::process_shortcode_atts( $atts );
		
		$atts = wp_parse_args( WP_Malscart::$product_form_fields, $atts );

		//error_log( __METHOD__.' shortcode attributes:'.print_r( $atts, true ));
		
		if ( is_object( $post ) ) $atts[ 'return' ] = get_bloginfo( 'url' ).'/'.$post->post_name;

		return self::buy_button_html( $atts );		

	}

	public function buy_button_html( $atts ) {
		
		// todo: make this into a templating class
		
		$html = array();	
		$atts[ 'button_text' ] = isset( $atts[ 'button_value' ] ) ? $atts[ 'button_value' ] : WP_Malscart::get_option( 'button_text' );

		// error_log( __METHOD__.' shortcode called with:'.print_r($atts,true));
		
		switch ( $atts[ 'button_style' ] ) :
			case 1 :
				$html[] = self::buy_button_form_head( $atts );
				$html[] = '<div class="'.self::$base_class.' '.self::$base_class.'-button-'.$atts[ 'button_style' ].'" style="width:'.$atts[ 'button_width' ].'" >';
				if ( isset( $atts[ 'product_options' ] ) ) $html[] = self::build_options_element( $atts[ 'product_options' ] );
				$html[] = self::buy_button_submit( $atts );
				$html[] = '</div>';
				$html[] = self::buy_button_form_end( $atts );
				break;
			case 2 :
				$html[] = self::buy_button_form_head( $atts );
				$html[] = '<div class="'.self::$base_class.' '.self::$base_class.'-button-'.$atts[ 'button_style' ].'" style="width:'.$atts[ 'button_width' ].'" >';
				$html[] = '<h4>'.$atts[ 'product[]' ].'<br/>$'.$atts[ 'price' ].'</h4>';
				if ( isset( $atts[ 'product_options' ] ) ) $html[] = self::build_options_element( $atts[ 'product_options' ] );
				$html[] = self::buy_button_submit( $atts );
				$html[] = '</div>';
				$html[] = self::buy_button_form_end( $atts );
				break;
			default :
				$html[] = '';
				error_log( __METHOD__.': invalid style selection: '.$atts[ 'button_style' ] );
		endswitch;
	
		//error_log( __METHOD__.': output array:'.htmlentities(print_r( $html,true )));
		return self::output( $html );

	}
	
/**
* build the product attributes element for a buy button
*
* the first one will be the selected one (unless we decide to allow the shortcode to determine that)
*
* the buy_button flag sets the options to use the products field instead of a separate field
*
*/
	private function build_options_element( Array $product_options ) {
		
		switch ( WP_Malscart::get_option( 'selector_preference' ) ) {
			
			case 'select':
				$element_function = 'dropdown_field';
				break;
			case 'radio':
			default :
				$element_function = 'radio_field';
				
		}
		
		$html = array();
		
		foreach( $product_options as $product_option => $attributes ) {
		
			$options_html = array();
			
			// title for the options set
			$options_html[] = self::build( array( 
												'tag'=>'p',
												'class'=>'option-title',
												'text'=> $attributes[ 'name' ] 
												) );

			// options select elements
			$options_html[] = self::$element_function( array(
												'title' => $attributes[ 'name' ],
												'name' => self::$buy_button ? 'product[]' : $product_option,
												'value' => current( $attributes[ 'options' ] ),
												'items' => self::values_to_keys( $attributes[ 'options' ] )
												) );

			// wrapper for the set
			$html[] = self::build( array( 
												'tag' => 'div',
												'class' => 'product-options product-options-'.$product_option,
												'text' => self::output( $options_html )
												) );

		}// product options
		
		return self::output( $html );
		
	}
	
/**
* utility to take an indexed array and make it of the form value => value
*/
	private function values_to_keys( Array $array ) {
		
		if ( self::isAssoc( $array ) ) return $array;
		
		$assoc_array = array();
		
		foreach( $array as $element )
			$assoc_array[ $element ] = $element;
			
		return $assoc_array;
		
	}
	
	// this just spits out the form open and the hidden fields for a buy button
	private function buy_button_form_head( $atts ) {
		
		if ( $atts[ 'demo' ] ) return ''; // skip the header when just showing the style	
		
		$server = WP_Malscart::get_option( 'cart_server' );
		$atts[ 'userid' ] = WP_Malscart::get_option( 'cust_id' );
		$button_text = $atts['button_text'];
		unset( $atts['button_text'], $atts[ 'button_style' ] );
		$html = array();
		
		$html[] = '<form action="'.$server.'/cf/add.cfm" method="post">';
		
		$html[] = self::add_hidden_fields( $atts );
		
		return self::output( $html );

	}
	
	private function add_hidden_fields( Array $fields ) {
		
		$html = array();
		
		foreach( $fields as $name => $value )
			$html[] = '<input name="'.$name.'" type="hidden" value="'.$value.'">';
			
		return self::output( $html );

	}

	private function buy_button_form_end( $atts ) {

		return $atts[ 'demo' ] ? '' : '</form>';

	}
	
	private function buy_button_submit( $atts ) {
				
		$html = array();
		
		$html[] = '<input type="submit" ';
		$html[] = $atts[ 'demo' ] ? 'onclick="return false;" ' : '';
		$html[] = ' value="' . $atts[ 'button_text' ].'">';
		
		return self::output( $html );
	
	}
	
	/**
	* process the shortcode arguments
	* mainly getting any product attributes and making htme into arrays
	*
	* product options are expected to be in the form: 'Option Name:Option 1,Option 2,Option 3'
	*/
	private function process_shortcode_atts( $arguments ) {
		
		$processed_atts = array();
		
		// this is so the shortcode can use the argument 'product'; we change it to the bracket form
		$arguments[ 'product[]' ] = $arguments[ 'product' ];
		unset( $arguments[ 'product' ] );
		$arguments[ 'button_width' ] = $arguments[ 'width' ];
		unset( $arguments[ 'width' ] );
		
		// transfer, process and get rid of reserved arguments, leaving only user arguments
		foreach( self::$standard_atts as $attribute ) {
			if ( isset( $arguments[ $attribute ] ) ) {

				// any processing of specific arguments can go here
				switch ( $attribute ) {

					case 'price' :

						// remove any denominations, commas or spaces added by the user.
						$number = preg_match( "|([0-9\.]+)|", str_replace( array( ',', ' '), '', $value ), $matches );
						$arguments[ $attribute ] = $matches[1];
						break;
						

					default:

				}

				$processed_atts[ $attribute ] = $arguments[ $attribute ];
				unset( $arguments[ $attribute ] );
			}
		}
			
		// process remaining attributes, looking for product options
		// 
		$processed_atts[ 'product_options' ] = array();

		foreach ( $arguments as $argument => $value ) :

			// since we don't know what product attribute name a shortcode will have
			// we look for one that has the proper formatting and create the product
			// attribute array with it
		
			// check the argument and capture the values
			if ( preg_match( "|([^:]+):(.+)|", $value, $matches ) > 0 ) {
	
				$name = $matches[1];
	
				// now get the option names
				$option_names = explode( ',', $matches[2] );
	
				$processed_atts[ 'product_options' ][ $argument ] = array(
																							'name' => $name,
																							'options' => $option_names
																							);

			}

		endforeach;
		
		//error_log( __METHOD__.' atts='.print_r( $processed_atts, true ));
		
		return $processed_atts;
		
	}
	
	/**
	* builds the HTML string for output
	*
	* given an associative array of arguments, this function returns an HTML text string
	*
	* @param array $args
	*	tag	the html tag to build
	*	text	the string to wrap the tag around
	*	class	the class attribute for the element
	*	...	any HTML element attribute
	*
	*/	
	private function build( $args ) {
		//error_log( __METHOD__.': raw $args= '.print_r( $args, true ));
		
		$atts = self::build_element_attributes( $args );
		
		$text = isset( $atts[ 'text' ] ) ? $atts[ 'text' ] : '';
		$tag = $atts[ 'tag' ];
		$attributes = implode( ' ', $atts[ 'attributes' ] ); 

		if ( in_array( $tag, self::$self_closers	) ) $pattern = '<%1$s %2$s />'; // self-closers don't wrap anything
		else $pattern = '<%1$s %2$s >%3$s</%1$s>';

		return sprintf( $pattern, $tag, $attributes, $text );

	}
	
	/**	
	* builds an attribute array for HTML elements 
	* and then processes the array into strings to build the HTML element with
	*/
	private function build_element_attributes( Array $attributes_array ) {

		$atts = array( 'attributes' => array() );// this is the blank array we will fill

		foreach( $attributes_array as $attribute => $value ) {
			
			switch ( $attribute ) {
				
				case 'tag':
				case 'button_style':
				case 'demo':
				case 'text':
				case 'items':
						if ( $attribute == 'items' && ! is_array( $value ) ) break;// this must be array or nothing
				case 'desc':
					// everything above falls through to here if valid
					$atts[ $attribute ] = $value;
					break;
					
				// the following all get dropped into the attributes subarray
				case 'button_text':
					$attribute = 'value';
				case 'class':
					$value = self::make_class_string( $value );
				case 'id':
				case 'name':
				case 'value':
				case 'style':
				default: // and any other attrubutes
				
					// make the attribute into a string to add to the tag as it's built
					$atts[ 'attributes' ][ $attribute ] = ' '.$attribute.'="'.$value.'" ';
					
			}
			
		}
		
		return $atts;
		
	}
	
	private function trim_array( Array $array ) {
				
		foreach( $array as &$value ) $value = trim( $value );
		unset( $value );// break the last reference
		
		return $array;
		
	}
	
	private function make_class_string( $input ) {
		
		// make the classes into a space-delimited string from different kinds of inputs
		switch ( true ) {
			case ( false !== strpos( $input, ',' ) ):
				$classes = explode( ',', $input );// comma-delimited string
				break;
			case ( false !== strpos( $input, ' ' ) ):
				$classes = explode( ' ', $input );// space-delimited string
				break;
			case ( is_array( $input ) ):
				$classes = $input;
				break;
			default: 
				$classes = array( $input );
		}
		$classes = self::trim_array( $classes );
		return implode( ' ', $classes );
		
	}
	
	// this function prepares a couple of special cases then merges the called array with the default array so all members are present
	//
	private function html_attributes_merge_defaults( $defaults, $attributes ) {
		foreach( $attributes as $attribute => &$value ) {
			if ( is_array( $value ) ) {
										foreach ( $value as $sub_attribute => &$sub_value ) {
											switch ( $sub_attribute ) {
												case 'class' :
													$sub_value = $defaults[ $attribute ][ $sub_attribute ] . ' ' . $sub_value;
													break;
												case 'style' :
													$sub_value = rtrim( $defaults[ $attribute ][ $sub_attribute ], ';' ) . ';' . $sub_value;
													break;
												default:
											}
										}
			}
		}
		
		unset( $value, $sub_value );
		
		return self::multimerge( $defaults, $attributes );
	}

	private function multimerge ( $array1, $array2 ) {
		
    static $this_function = __METHOD__;
		if (is_array($array2) && count($array2)) {
      foreach ($array2 as $k => $v) {
        if (is_array($v) && count($v)) {
          $array1[$k] = self::multimerge($array1[$k], $v);
        } else {
          $array1[$k] = $v;
        }
      }
    } else {
      $array1 = $array2;
    }

    return $array1;
		
  }

/*
	output the HTML by collapsing an array
	or just print a string
*/
	private function output( $html ) {
		
		if ( ! count( $html ) ) $html = array( $html );
		
		return implode( "\r", $html );
		
	}



} // class def
?>
